package com.sun.imageio.plugins.wbmp;

import java.io.IOException;
import java.awt.image.WritableRaster;
import java.awt.image.SampleModel;
import java.awt.image.RenderedImage;
import java.awt.image.IndexColorModel;
import java.awt.image.DataBufferByte;
import java.awt.image.Raster;
import java.awt.Point;
import java.awt.image.MultiPixelPackedSampleModel;
import java.awt.Rectangle;
import javax.imageio.IIOImage;
import javax.imageio.ImageTypeSpecifier;
import javax.imageio.metadata.IIOMetadata;
import javax.imageio.ImageWriteParam;
import com.sun.imageio.plugins.common.I18N;
import javax.imageio.spi.ImageWriterSpi;
import javax.imageio.stream.ImageOutputStream;
import javax.imageio.ImageWriter;

public class WBMPImageWriter extends ImageWriter
{
    private ImageOutputStream stream;
    
    private static int getNumBits(int n) {
        int n2 = 32;
        for (int n3 = Integer.MIN_VALUE; n3 != 0 && (n & n3) == 0x0; n3 >>>= 1) {
            --n2;
        }
        return n2;
    }
    
    private static byte[] intToMultiByte(int n) {
        byte[] array = new byte[(getNumBits(n) + 6) / 7];
        for (int n2 = array.length - 1, i = 0; i <= n2; ++i) {
            array[i] = (byte)(n >>> (n2 - i) * 7 & 0x7F);
            if (i != n2) {
                int n3 = i;
                array[n3] |= 0xFFFFFF80;
            }
        }
        return array;
    }
    
    public WBMPImageWriter(ImageWriterSpi originatingProvider) {
        super(originatingProvider);
        this.stream = null;
    }
    
    @Override
    public void setOutput(Object output) {
        super.setOutput(output);
        if (output != null) {
            if (!(output instanceof ImageOutputStream)) {
                throw new IllegalArgumentException(I18N.getString("WBMPImageWriter"));
            }
            this.stream = (ImageOutputStream)output;
        }
        else {
            this.stream = null;
        }
    }
    
    @Override
    public IIOMetadata getDefaultStreamMetadata(ImageWriteParam imageWriteParam) {
        return null;
    }
    
    @Override
    public IIOMetadata getDefaultImageMetadata(ImageTypeSpecifier imageTypeSpecifier, ImageWriteParam imageWriteParam) {
        WBMPMetadata wbmpMetadata = new WBMPMetadata();
        wbmpMetadata.wbmpType = 0;
        return wbmpMetadata;
    }
    
    @Override
    public IIOMetadata convertStreamMetadata(IIOMetadata iioMetadata, ImageWriteParam imageWriteParam) {
        return null;
    }
    
    @Override
    public IIOMetadata convertImageMetadata(IIOMetadata iioMetadata, ImageTypeSpecifier imageTypeSpecifier, ImageWriteParam imageWriteParam) {
        return null;
    }
    
    @Override
    public boolean canWriteRasters() {
        return true;
    }
    
    @Override
    public void write(IIOMetadata iioMetadata, IIOImage iioImage, ImageWriteParam defaultWriteParam) throws IOException {
        if (this.stream == null) {
            throw new IllegalStateException(I18N.getString("WBMPImageWriter3"));
        }
        if (iioImage == null) {
            throw new IllegalArgumentException(I18N.getString("WBMPImageWriter4"));
        }
        this.clearAbortRequest();
        this.processImageStarted(0);
        if (defaultWriteParam == null) {
            defaultWriteParam = this.getDefaultWriteParam();
        }
        RenderedImage renderedImage = null;
        int hasRaster = iioImage.hasRaster();
        Rectangle sourceRegion = defaultWriteParam.getSourceRegion();
        SampleModel sampleModel;
        WritableRaster rect;
        if (!(hasRaster == 0)) {
            rect = iioImage.getRaster();
            sampleModel = rect.getSampleModel();
        }
        else {
            renderedImage = iioImage.getRenderedImage();
            sampleModel = renderedImage.getSampleModel();
            rect = renderedImage.getData();
        }
        this.checkSampleModel(sampleModel);
        Rectangle obj;
        if (sourceRegion == null) {
            obj = rect.getBounds();
        }
        else {
            obj = sourceRegion.intersection(rect.getBounds());
        }
        if (obj.isEmpty()) {
            throw new RuntimeException(I18N.getString("WBMPImageWriter1"));
        }
        int sourceXSubsampling = defaultWriteParam.getSourceXSubsampling();
        int sourceYSubsampling = defaultWriteParam.getSourceYSubsampling();
        int subsamplingXOffset = defaultWriteParam.getSubsamplingXOffset();
        int subsamplingYOffset = defaultWriteParam.getSubsamplingYOffset();
        obj.translate(subsamplingXOffset, subsamplingYOffset);
        obj.width -= subsamplingXOffset;
        obj.height -= subsamplingYOffset;
        int childMinX = obj.x / sourceXSubsampling;
        int i = obj.y / sourceYSubsampling;
        int width = (obj.width + sourceXSubsampling - 1) / sourceXSubsampling;
        int height = (obj.height + sourceYSubsampling - 1) / sourceYSubsampling;
        Rectangle o = new Rectangle(childMinX, i, width, height);
        SampleModel compatibleSampleModel = sampleModel.createCompatibleSampleModel(width, height);
        if (compatibleSampleModel.getDataType() != 0 || !(compatibleSampleModel instanceof MultiPixelPackedSampleModel) || ((MultiPixelPackedSampleModel)compatibleSampleModel).getDataBitOffset() != 0) {
            compatibleSampleModel = new MultiPixelPackedSampleModel(0, width, height, 1, width + 7 >> 3, 0);
        }
        if (!o.equals(obj)) {
            if (sourceXSubsampling == 1 && sourceYSubsampling == 1) {
                rect = rect.createChild(rect.getMinX(), rect.getMinY(), width, height, childMinX, i, (int[])null);
            }
            else {
                WritableRaster writableRaster = Raster.createWritableRaster(compatibleSampleModel, new Point(childMinX, i));
                byte[] data = ((DataBufferByte)writableRaster.getDataBuffer()).getData();
                for (int y = obj.y, n = 0; i < i + height; n += width + 7 >> 3, ++i, y += sourceYSubsampling) {
                    for (int j = 0, x = obj.x; j < width; ++j, x += sourceXSubsampling) {
                        int sample = rect.getSample(x, y, 0);
                        int n2 = n + (j >> 3);
                        data[n2] |= sample << 7 - (j & 0x7);
                    }
                }
                rect = writableRaster;
            }
        }
        if (!compatibleSampleModel.equals(rect.getSampleModel())) {
            WritableRaster writableRaster2 = Raster.createWritableRaster(compatibleSampleModel, new Point(rect.getMinX(), rect.getMinY()));
            writableRaster2.setRect(rect);
            rect = writableRaster2;
        }
        int n3 = 0;
        if (hasRaster == 0 && renderedImage.getColorModel() instanceof IndexColorModel) {
            IndexColorModel indexColorModel = (IndexColorModel)renderedImage.getColorModel();
            n3 = (indexColorModel.getRed(0) > indexColorModel.getRed(1));
        }
        int scanlineStride = ((MultiPixelPackedSampleModel)compatibleSampleModel).getScanlineStride();
        int n4 = (width + 7) / 8;
        byte[] data2 = ((DataBufferByte)rect.getDataBuffer()).getData();
        this.stream.write(0);
        this.stream.write(0);
        this.stream.write(intToMultiByte(width));
        this.stream.write(intToMultiByte(height));
        if (n3 == 0 && scanlineStride == n4) {
            this.stream.write(data2, 0, height * n4);
            this.processImageProgress(100.0f);
        }
        else {
            int n5 = 0;
            if (n3 == 0) {
                for (int n6 = 0; n6 < height && !this.abortRequested(); ++n6) {
                    this.stream.write(data2, n5, n4);
                    n5 += scanlineStride;
                    this.processImageProgress(100.0f * (float)n6 / (float)height);
                }
            }
            else {
                byte[] array = new byte[n4];
                int k = 0;
                while (k < height) {
                    if (this.abortRequested()) {
                        break;
                    }
                    for (int l = 0; l < n4; ++l) {
                        array[l] = (byte)~data2[l + n5];
                    }
                    this.stream.write(array, 0, n4);
                    n5 += scanlineStride;
                    this.processImageProgress(100.0f * (float)k / (float)height);
                    ++k;
                }
            }
        }
        if (this.abortRequested()) {
            this.processWriteAborted();
        }
        else {
            this.processImageComplete();
            this.stream.flushBefore(this.stream.getStreamPosition());
        }
        return;
    }
    
    @Override
    public void reset() {
        super.reset();
        this.stream = null;
    }
    
    private void checkSampleModel(SampleModel sampleModel) {
        int dataType = sampleModel.getDataType();
        if (dataType < 0 || dataType > 3 || sampleModel.getNumBands() != 1 || sampleModel.getSampleSize(0) != 1) {
            throw new IllegalArgumentException(I18N.getString("WBMPImageWriter2"));
        }
        return;
    }
}
